@@ -15,12 +15,10 @@ from .constants import NULL_TAG_VAL
 from .constants import SERVICE_TAG_KEY
 from .constants import SHARD_TAG_KEY
 
-logging.basicConfig(level=logging.INFO)
-LOGGER = logging.getLogger(__name__)
-
 
 # pylint: disable=E0012,R0205
 # pylint: disable=too-few-public-methods
+# pylint: disable=too-many-instance-attributes
 class HeartbeaterService(object):
     """Service that periodically reports component heartbeats to Wavefront."""
 
@@ -38,25 +36,36 @@ class HeartbeaterService(object):
         self.source = source
         self.reporting_interval_seconds = 60 * 5
         self.heartbeat_metric_tags_list = []
+        self.custom_tags_set = set()
         if isinstance(components, str):
             components = [components]
         for component in components:
-            self.heartbeat_metric_tags_list.append(
-                {
-                    APPLICATION_TAG_KEY: application_tags.application,
-                    CLUSTER_TAG_KEY: application_tags.cluster or NULL_TAG_VAL,
-                    SERVICE_TAG_KEY: application_tags.service,
-                    SHARD_TAG_KEY: application_tags.shard or NULL_TAG_VAL,
-                    COMPONENT_TAG_KEY: component
-                }
-            )
+            metric_tags = {
+                APPLICATION_TAG_KEY: application_tags.application,
+                CLUSTER_TAG_KEY: application_tags.cluster or NULL_TAG_VAL,
+                SERVICE_TAG_KEY: application_tags.service,
+                SHARD_TAG_KEY: application_tags.shard or NULL_TAG_VAL,
+                COMPONENT_TAG_KEY: component
+            }
+            if application_tags.custom_tags:
+                metric_tags.update(dict(application_tags.custom_tags))
+            self.heartbeat_metric_tags_list.append(metric_tags)
         self._closed = False
         self._timer = None
         self._run()
 
+    def report_custom_tags(self, custom_tags):
+        """
+        Append custom tags for heartbeat reporting.
+
+        @param custom_tags: dict of custom tags.
+        """
+        self.custom_tags_set.add(custom_tags)
+
     def _schedule_timer(self):
         self._timer = threading.Timer(self.reporting_interval_seconds,
                                       self._run)
+        self._timer.daemon = True
         self._timer.start()
 
     def _run(self):
@@ -72,10 +81,16 @@ class HeartbeaterService(object):
                 self.wavefront_client.send_metric(HEART_BEAT_METRIC, 1.0,
                                                   time.time(), self.source,
                                                   heartbeat)
+            while self.custom_tags_set:
+                self.wavefront_client.send_metric(HEART_BEAT_METRIC, 1.0,
+                                                  time.time(), self.source,
+                                                  self.custom_tags_set.pop())
+
         # pylint: disable=broad-except,fixme
         # TODO: Please make sure we catch more specific exception here.
         except Exception:
-            LOGGER.warning('Can not report %s to wavefront', HEART_BEAT_METRIC)
+            logging.warning('Can not report %s to wavefront',
+                            HEART_BEAT_METRIC)
 
     def close(self):
         """Cancel the timer."""
