@@ -7,6 +7,7 @@
 from __future__ import absolute_import
 
 import logging
+import socket
 import threading
 
 import requests
@@ -16,12 +17,9 @@ try:
 except ImportError:
     import Queue as queue  # noqa
 
-from . import common
 from . import entities
-from .common import connection_handler
-from .common.utils import gzip_compress
-
-LOGGER = logging.getLogger('wavefront_sdk.WavefrontDirectClient')
+from .common import connection_handler, constants, utils
+from .common.metrics import registry
 
 
 # pylint: disable=too-many-instance-attributes
@@ -64,7 +62,7 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         self._max_queue_size = max_queue_size
         self._batch_size = batch_size
         self._flush_interval_seconds = flush_interval_seconds
-        self._default_source = 'wavefrontDirectSender'
+        self._default_source = socket.gethostname() or 'unknown'
         self._metrics_buffer = queue.Queue(max_queue_size)
         self._histograms_buffer = queue.Queue(max_queue_size)
         self._tracing_spans_buffer = queue.Queue(max_queue_size)
@@ -77,7 +75,67 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         self._timer = None
         self._schedule_timer()
 
-    def _report(self, points, data_format):
+        self._sdk_metrics_registry = registry.WavefrontSdkMetricsRegistry(
+            wf_metric_sender=self,
+            prefix='{}.core.sender.direct'.format(constants.SDK_METRIC_PREFIX))
+
+        self._sdk_metrics_registry.new_gauge(
+            'points.queue.size', self._metrics_buffer.qsize)
+        self._sdk_metrics_registry.new_gauge(
+            'points.queue.remaining_capacity',
+            remaining_capacity_getter(self._metrics_buffer))
+        self._points_valid = self._sdk_metrics_registry.new_counter(
+            'points.valid')
+        self._points_invalid = self._sdk_metrics_registry.new_counter(
+            'points.invalid')
+        self._points_dropped = self._sdk_metrics_registry.new_counter(
+            'points.dropped')
+        self._points_report_errors = self._sdk_metrics_registry.new_counter(
+            'points.report.errors')
+
+        self._sdk_metrics_registry.new_gauge(
+            'histograms.queue.size', self._histograms_buffer.qsize)
+        self._sdk_metrics_registry.new_gauge(
+            'histograms.queue.remaining_capacity',
+            remaining_capacity_getter(self._histograms_buffer))
+        self._histograms_valid = self._sdk_metrics_registry.new_counter(
+            'histograms.valid')
+        self._histograms_invalid = self._sdk_metrics_registry.new_counter(
+            'histograms.invalid')
+        self._histograms_dropped = self._sdk_metrics_registry.new_counter(
+            'histograms.dropped')
+        self._histograms_report_errors = (
+            self._sdk_metrics_registry.new_counter('histograms.report.errors'))
+
+        self._sdk_metrics_registry.new_gauge(
+            'spans.queue.size', self._tracing_spans_buffer.qsize)
+        self._sdk_metrics_registry.new_gauge(
+            'spans.queue.remaining_capacity',
+            remaining_capacity_getter(self._tracing_spans_buffer))
+        self._spans_valid = self._sdk_metrics_registry.new_counter(
+            'spans.valid')
+        self._spans_invalid = self._sdk_metrics_registry.new_counter(
+            'spans.invalid')
+        self._spans_dropped = self._sdk_metrics_registry.new_counter(
+            'spans.dropped')
+        self._spans_report_errors = self._sdk_metrics_registry.new_counter(
+            'spans.report.errors')
+
+        self._sdk_metrics_registry.new_gauge(
+            'span_logs.queue.size', self._spans_log_buffer.qsize)
+        self._sdk_metrics_registry.new_gauge(
+            'span_logs.queue.remaining_capacity',
+            remaining_capacity_getter(self._spans_log_buffer))
+        self._span_logs_valid = self._sdk_metrics_registry.new_counter(
+            'span_logs.valid')
+        self._span_logs_invalid = self._sdk_metrics_registry.new_counter(
+            'span_logs.invalid')
+        self._span_logs_dropped = self._sdk_metrics_registry.new_counter(
+            'span_logs.dropped')
+        self._span_logs_report_errors = self._sdk_metrics_registry.new_counter(
+            'span_logs.report.errors')
+
+    def _report(self, points, data_format, entity_prefix, report_errors):
         r"""One api call sending one given string data.
 
         @param points: List of data in string format, concat by '\n'
@@ -87,16 +145,20 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         """
         try:
             params = {'f': data_format}
-            compressed_data = gzip_compress(points.encode('utf-8'))
+            compressed_data = utils.gzip_compress(points.encode('utf-8'))
             response = requests.post(self.server + '/report', params=params,
                                      headers=self._headers,
                                      data=compressed_data)
+            self._sdk_metrics_registry.new_counter(
+                '{}.report.{}'.format(entity_prefix,
+                                      response.status_code)).inc()
             response.raise_for_status()
         except Exception as error:
-            self.increment_failure_count()
+            report_errors.inc()
             raise error
 
-    def _batch_report(self, batch_line_data, data_format):
+    def _batch_report(self, batch_line_data, data_format, entity_prefix,
+                      report_errors):
         """One api call sending one given list of data.
 
         @param batch_line_data: List of data to be sent
@@ -105,17 +167,20 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @type data_format: str
         """
         # Split data into chunks, each with the size of given batch_size
-        for batch in common.utils.chunks(batch_line_data, self._batch_size):
+        for batch in utils.chunks(batch_line_data, self._batch_size):
             # report once per batch
             try:
-                self._report('\n'.join(batch) + '\n', data_format)
+                self._report('\n'.join(batch) + '\n', data_format,
+                             entity_prefix, report_errors)
             # pylint: disable=broad-except,fixme
             # TODO: Please replace a generic Exception with a specific one.
             except Exception as error:
-                LOGGER.error('Failed to report %s data points to wavefront %s',
-                             data_format, error)
+                logging.error(
+                    'Failed to report %s data points to wavefront %s',
+                    data_format, error)
 
-    def _internal_flush(self, data_buffer, data_format):
+    def _internal_flush(self, data_buffer, data_format, entity_prefix,
+                        report_errors):
         """Get all data from one data buffer to a list, and report that list.
 
         @param data_buffer: Data buffer to be flush and sent
@@ -128,13 +193,14 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         while size > 0 and not data_buffer.empty():
             data.append(data_buffer.get())
             size -= 1
-        self._batch_report(data, data_format)
+        self._batch_report(data, data_format, entity_prefix, report_errors)
 
     def _schedule_timer(self):
         # Flush every 5 secs by default
         if not self._closed:
             self._timer = threading.Timer(self._flush_interval_seconds,
                                           self._flush)
+            self._timer.daemon = True
             self._timer.start()
 
     def _flush(self):
@@ -149,13 +215,17 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
     def flush_now(self):
         """Flush all the data buffer immediately."""
         self._internal_flush(self._metrics_buffer,
-                             self.WAVEFRONT_METRIC_FORMAT)
+                             self.WAVEFRONT_METRIC_FORMAT, 'points',
+                             self._points_report_errors)
         self._internal_flush(self._histograms_buffer,
-                             self.WAVEFRONT_HISTOGRAM_FORMAT)
+                             self.WAVEFRONT_HISTOGRAM_FORMAT, 'histograms',
+                             self._histograms_report_errors)
         self._internal_flush(self._tracing_spans_buffer,
-                             self.WAVEFRONT_TRACING_SPAN_FORMAT)
+                             self.WAVEFRONT_TRACING_SPAN_FORMAT, 'spans',
+                             self._spans_report_errors)
         self._internal_flush(self._spans_log_buffer,
-                             self.WAVEFRONT_SPAN_LOG_FORMAT)
+                             self.WAVEFRONT_SPAN_LOG_FORMAT, 'span_logs',
+                             self._span_logs_report_errors)
 
     def close(self):
         """Flush all buffer before close the client."""
@@ -164,6 +234,7 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
             self._closed = True
             if self._timer is not None:
                 self._timer.cancel()
+        self._sdk_metrics_registry.close()
 
     # pylint: disable=too-many-arguments
 
@@ -186,9 +257,18 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param tags: Tags
         @type tags: dict
         """
-        line_data = common.utils.metric_to_line_data(
-            name, value, timestamp, source, tags, self._default_source)
-        self._metrics_buffer.put(line_data)
+        try:
+            line_data = utils.metric_to_line_data(
+                name, value, timestamp, source, tags, self._default_source)
+            self._points_valid.inc()
+        except ValueError as error:
+            self._points_invalid.inc()
+            raise error
+        try:
+            self._metrics_buffer.put(line_data)
+        except queue.Full as error:
+            self._points_dropped.inc()
+            raise error
 
     def send_metric_now(self, metrics):
         """Send a list of metrics immediately.
@@ -199,7 +279,8 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param metrics: List of string spans data
         @type metrics: list[str]
         """
-        self._batch_report(metrics, self.WAVEFRONT_METRIC_FORMAT)
+        self._batch_report(metrics, self.WAVEFRONT_METRIC_FORMAT, 'points',
+                           self._points_report_errors)
 
     # pylint: disable=too-many-arguments
 
@@ -226,10 +307,19 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param tags: Tags
         @type tags: dict
         """
-        line_data = common.utils.histogram_to_line_data(
-            name, centroids, histogram_granularities, timestamp, source, tags,
-            self._default_source)
-        self._histograms_buffer.put(line_data)
+        try:
+            line_data = utils.histogram_to_line_data(
+                name, centroids, histogram_granularities, timestamp,
+                source, tags, self._default_source)
+            self._histograms_valid.inc()
+        except ValueError as error:
+            self._histograms_invalid.inc()
+            raise error
+        try:
+            self._histograms_buffer.put(line_data)
+        except queue.Full as error:
+            self._histograms_dropped.inc()
+            raise error
 
     def send_distribution_now(self, distributions):
         """Send a list of distribution immediately.
@@ -240,7 +330,8 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param distributions: List of string spans data
         @type distributions: list[str]
         """
-        self._batch_report(distributions, self.WAVEFRONT_HISTOGRAM_FORMAT)
+        self._batch_report(distributions, self.WAVEFRONT_HISTOGRAM_FORMAT,
+                           'histograms', self._histograms_report_errors)
 
     # pylint: disable=too-many-arguments
 
@@ -278,14 +369,32 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @type tags: list
         @param span_logs: Span Log
         """
-        line_data = common.utils.tracing_span_to_line_data(
-            name, start_millis, duration_millis, source, trace_id, span_id,
-            parents, follows_from, tags, span_logs, self._default_source)
-        self._tracing_spans_buffer.put(line_data)
+        try:
+            line_data = utils.tracing_span_to_line_data(
+                name, start_millis, duration_millis, source, trace_id, span_id,
+                parents, follows_from, tags, span_logs, self._default_source)
+            self._spans_valid.inc()
+        except ValueError as error:
+            self._spans_invalid.inc()
+            raise error
+        try:
+            self._tracing_spans_buffer.put(line_data)
+        except queue.Full as error:
+            self._spans_dropped.inc()
+            raise error
         if span_logs:
-            span_log_line_data = common.utils.span_log_to_line_data(
-                trace_id, span_id, span_logs)
-            self._spans_log_buffer.put(span_log_line_data)
+            try:
+                line_data = utils.span_log_to_line_data(
+                    trace_id, span_id, span_logs)
+                self._span_logs_valid.inc()
+            except ValueError as error:
+                self._span_logs_invalid.inc()
+                raise error
+            try:
+                self._spans_log_buffer.put(line_data)
+            except queue.Full as error:
+                self._span_logs_dropped.inc()
+                raise error
 
     def send_span_now(self, spans):
         """
@@ -297,7 +406,8 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param spans: List of string spans data
         @type spans: list[str]
         """
-        self._batch_report(spans, self.WAVEFRONT_TRACING_SPAN_FORMAT)
+        self._batch_report(spans, self.WAVEFRONT_TRACING_SPAN_FORMAT, 'spans',
+                           self._spans_report_errors)
 
     def send_span_log_now(self, span_logs):
         """
@@ -309,4 +419,21 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param span_logs: List of string span logs data
         @type span_logs: list[str]
         """
-        self._batch_report(span_logs, self.WAVEFRONT_SPAN_LOG_FORMAT)
+        self._batch_report(span_logs, self.WAVEFRONT_SPAN_LOG_FORMAT,
+                           'span_logs', self._span_logs_report_errors)
+
+    def get_failure_count(self):
+        """Get failure count for one connection."""
+        return (self._points_report_errors + self._histograms_report_errors +
+                self._spans_report_errors + self._span_logs_report_errors)
+
+
+def remaining_capacity_getter(buf):
+    """Get remaining capacity of given queue.
+
+    @param buf: Input Buffer Queue.
+    @type buf: queue.Queue
+    @return: Remaining capacity of the queue.
+    @rtype: int
+    """
+    return lambda: buf.maxsize - buf.qsize()
